<?php

namespace Drupal\pdb_file_viewer\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class PDBFileViewerSettingsForm.
 */
class PDBFileViewerSettingsForm extends ConfigFormBase {

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The extension list module service.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected ModuleExtensionList $extensionListBuilder;

  /**
   * Constructs a new PDBFileViewerSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration form configuration.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Extension\ModuleExtensionList $extensionListBuilder
   *   The extension list module service.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              MessengerInterface $messenger,
                              ModuleExtensionList $extensionListBuilder) {
    parent::__construct($config_factory);
    $this->messenger = $messenger;
    $this->extensionListBuilder = $extensionListBuilder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('config.factory'),
          $container->get('messenger'),
          $container->get('extension.list.module'),
      );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pdb_file_viewer_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'pdb_file_viewer.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('pdb_file_viewer.settings');

    $form['use_cdn'] = [
      '#prefix' => '<Strong>NGL Library Location</Strong>',
      '#type' => 'radios',
      '#default_value' => $config->get('use_cdn'),
      '#options' => [
        'cdn' => 'cdn (https://unpkg.com/ngl@0.10.4/dist/ngl.js)',
        'local' => 'local',
      ],
    ];

    $form['test_cdn'] = [
      '#prefix' => '<div>&nbsp;</div>',
      '#type' => 'submit',
      '#value' => 'Test cdn',
      '#submit' => ['::testCdn'],
      '#states' => [
        'visible' => [
          ':input[name=use_cdn]' => [
            'value' => 'cdn',
          ],
        ],
      ],
    ];

    $form['test_local'] = [
      '#prefix' => '<div>&nbsp;</div>',
      '#type' => 'submit',
      '#value' => 'Test local library',
      '#submit' => ['::testLocal'],
      '#states' => [
        'visible' => [
          ':input[name=use_cdn]' => [
            'value' => 'local',
          ],
        ],
      ],
    ];

    $option = $config->get('use_cdn');

    if ($option === 'cdn') {
      $url = 'https://unpkg.com/ngl@0.10.4/dist/ngl.js';
      $headers = get_headers($url);
      stripos($headers[0], "200 OK") ? $this->messenger->addMessage('The CDN is reachable.') : $this->messenger->addError('The CDN is not reachable.');
    }

    if ($option === 'local') {
      $modulePath = $this->extensionListBuilder->getPath('pdb_file_viewer');
      $file = $modulePath . '/libraries/ngl.js';

      if (file_exists($file)) {
        $this->messenger->addMessage('ngl.js exists.');
      }
      else {
        $this->messenger->addError($file . ' does not exist.');
      }
    }

    $allow_all_extensions = $config->get('allow_all_extensions');
    $form['allow_all_extensions'] = [
      '#type' => 'checkbox',
      '#description' => "Check this if you want to render all files with PDB File Viewer.",
      '#title' => 'Allow all extensions',
      '#default_value' => $allow_all_extensions,
    ];

    $supported_extensions = $config->get('supported_extensions');
    $form['supported_extensions'] = [
      '#title' => 'Supported Extensions',
      '#type' => 'textarea',
      '#default_value' => $supported_extensions,
      '#states' => [
        'visible' => [
          ':input[name=allow_all_extensions]' => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $form['reset'] = [
      '#type' => 'submit',
      '#value' => 'Restore default supported extensions',
      '#submit' => ['::resetSubmit'],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Test CDN.
   */
  public function testCdn(array &$form, FormStateInterface $form_state) {
    $url = 'https://unpkg.com/ngl@0.10.4/dist/ngl.js';
    $headers = get_headers($url);
    stripos($headers[0], "200 OK") ? $this->messenger->addMessage('The CDN is reachable.') : $this->messenger->addError('The CDN is not reachable.');
  }

  /**
   * Test the local library.
   */
  public function testLocal(array &$form, FormStateInterface $form_state) {
    $modulePath = $this->extensionListBuilder->getPath('pdb_file_viewer');
    $file = $modulePath . '/libraries/ngl.js';

    if (file_exists($file)) {
      $this->messenger->addMessage('ngl.js exists.');
    }
    else {
      $this->messenger->addError($file . ' does not exist.');
    }
  }

  /**
   * Submit the reset.
   */
  public function resetSubmit(array &$form, FormStateInterface $form_state) {
    $this->config('pdb_file_viewer.settings')
      ->set('use_cdn', $this->config('pdb_file_viewer.settings.default')->get('use_cdn'))
      ->set('supported_extensions', $this->config('pdb_file_viewer.settings.default')->get('supported_extensions'))
      ->set('allow_all_extensions', $this->config('pdb_file_viewer.settings.default')->get('allow_all_extensions'))
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('pdb_file_viewer.settings')
      ->set('use_cdn', $form_state->getValue('use_cdn'))
      ->set('supported_extensions', $form_state->getValue('supported_extensions'))
      ->set('allow_all_extensions', $form_state->getValue('allow_all_extensions'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
